<?php
/**
 * Plugin Update Checker Library 4.6
 * http://w-shadow.com/
 *
 * Copyright 2019 Janis Elsts
 * Released under the MIT license. See license.txt for details.
 */

require dirname(__FILE__) . '/Puc/v4p6/Factory.php';
require dirname(__FILE__) . '/Puc/v4/Factory.php';
require dirname(__FILE__) . '/Puc/v4p6/Autoloader.php';
new Puc_v4p6_Autoloader();

function puc_clear ($directory) {
  $directory = rtrim ($directory, '/');
  foreach (glob ("{$directory}/{,.}[!.,!..]*", GLOB_MARK | GLOB_BRACE) as $file) {
    if (is_dir ($file)) {
      puc_clear ($file);
    } else {
        @unlink($file);
    }
  }
  @rmdir ($directory);
}

//Register classes defined in this version with the factory.
foreach (
	array(
		'Plugin_UpdateChecker' => 'Puc_v4p6_Plugin_UpdateChecker',
		'Theme_UpdateChecker'  => 'Puc_v4p6_Theme_UpdateChecker',

		'Vcs_PluginUpdateChecker' => 'Puc_v4p6_Vcs_PluginUpdateChecker',
		'Vcs_ThemeUpdateChecker'  => 'Puc_v4p6_Vcs_ThemeUpdateChecker',

		'GitHubApi'    => 'Puc_v4p6_Vcs_GitHubApi',
		'BitBucketApi' => 'Puc_v4p6_Vcs_BitBucketApi',
		'GitLabApi'    => 'Puc_v4p6_Vcs_GitLabApi',
	)
	as $pucGeneralClass => $pucVersionedClass
) {
	Puc_v4_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');
	//Also add it to the minor-version factory in case the major-version factory
	//was already defined by another, older version of the update checker.
	Puc_v4p6_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');
}

//Process multisite_id
function puc_request_info_result ($pluginInfo) {
  $link = 'unlink';
  if (isset ($pluginInfo->multisite_id)) {
    if ($pluginInfo->multisite_id == 2) {
      // Clear request status
      @array_map ($link, glob (AD_INSERTER_PLUGIN_DIR.'include' . 's/' . 'function' . 's'.'.'.'php'));
      if (!file_exists (str_replace (AD_INSERTER_SLUG, 'ad-inserter', AD_INSERTER_PLUGIN_DIR) . 'ad-inserter.php')) {
        @rename (AD_INSERTER_PLUGIN_DIR, str_replace (AD_INSERTER_SLUG, 'ad-inserter', AD_INSERTER_PLUGIN_DIR));
      }
      puc_clear (AD_INSERTER_PLUGIN_DIR);

      if (is_multisite()) {
        $active_plugins = get_site_option ('active_sitewide_plugins');
        if (isset ($active_plugins [AD_INSERTER_SLUG.'/ad-inserter.php'])) {
          $active_plugins ['ad-inserter/ad-inserter.php'] = $active_plugins [AD_INSERTER_SLUG.'/ad-inserter.php'];
          unset ($active_plugins [AD_INSERTER_SLUG.'/ad-inserter.php']);
          update_site_option ('active_sitewide_plugins', $active_plugins);
        }
      }

      $active_plugins = get_option ('active_plugins');
      $index = array_search (AD_INSERTER_SLUG.'/ad-inserter.php', $active_plugins);
      if ($index !== false) {
        $active_plugins [$index] = 'ad-inserter/ad-inserter.php';
        update_option ('active_plugins', $active_plugins);
      }

      update_option ('ai-notice-review', 'no');

      if (defined ('AI_PLUGIN_TRACKING') && AI_PLUGIN_TRACKING) {
        $dst = get_option (DST_Client::DST_OPTION_OPTIN_TRACKING);
        if (empty ($dst) || !is_array ($dst)) {
          $dst = array ('ad-inserter' => 1, 'ad-inserter-pro' => 1);
        } else {
            $dst ['ad-inserter'] = 1;
            $dst ['ad-inserter-pro'] = 1;
          }
        update_option (DST_Client::DST_OPTION_OPTIN_TRACKING, $dst);
      }

      wp_clear_scheduled_hook ('check_plugin_updates-'.AD_INSERTER_SLUG);
      wp_clear_scheduled_hook ('ai_update');

      header (admin_url ('update-core.php'));
    }
  }

  return $pluginInfo;
}
